require "LinkAgainstFinder/version"
require 'LinkAgainstFinder/objdump'

module LinkAgainstFinder

  # set the develop dir for objdump, you can use this to chose version
  # /Applications/Xcode11.app/Contents/Developer
  def self.setDevelopDir(path, raw = false)
    if !raw
      path = "DEVELOPER_DIR=#{path}"
    end
    $LinkAgainstFinder_develop_dir_env = path
  end

  class Finder

    public
    # @return [Lib]
    def calculateEachOtherDependencyRelations(binaries)
      calculateDependencyRelations(binaries, binaries)
    end


    # @return [Lib]
    def calculateDependencyRelations(targetBins, potentialBins)

      cache = {}
      generate_lib = Proc.new do |path|
        lib = cache[path]
        next lib unless lib.nil?

        lib = Lib.new
        lib.path = path
        lib.symbolTable = Objdump.new(path).getSymbolTable
        lib.dependencies = Set.new
        cache[path] = lib
        lib
      end

      targets = targetBins.map(&generate_lib)
      potentials = potentialBins.map(&generate_lib)

      targets.each do |lib|
        lib.symbolTable.externalSymbols.each do |symbol|
          dep = potentials.find do |otherLib|
            otherLib.symbolTable.internalSymbols.include?(symbol)
          end
          lib.dependencies.add(dep) unless dep.nil?
        end
      end

      return targets
    end

    class Lib
      attr_accessor :path
      attr_accessor :symbolTable
      attr_accessor :dependencies

      def to_s
        <<-TEXT
#{path}:
    #{dependencies.map{|l| "-> #{l.path}"}.join("\n")}
        TEXT
      end
    end

  end
end



